#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include "r2conf.h"
#include "r2serial.h"
#include "civ.h"
#include "r2radio.h"

extern int value[256];

int CIVCommand::writeToSerialPort() {
	putR2SerialByte(0xfe);
	putR2SerialByte(0xfe);
	putR2SerialByte(0xee); //toAddr
	putR2SerialByte(0xef); //fmAddr
	putR2SerialByte(cmd);
//	putR2SerialByte(sub);
	for(unsigned int i = 0; i < numDataBytes; i++)
		putR2SerialByte(data[i]);
	putR2SerialByte(0xfd);

	return 0;
}

int CIVCommand::writeToVlaskiFile(FILE * fp) {
	fputc(0xfe, fp);
	fputc(0xfe, fp);
	fputc(0xee, fp); //toAddr
	fputc(0xef, fp); //fmAddr
	fputc(cmd, fp);
//	fputc(sub, fp);
	for(unsigned int i = 0; i < numDataBytes; i++)
		fputc(data[i], fp);
	fputc(0xfd, fp);

	return 0;


}


int CIVCommand::readFromSerialPort(){

	byte ch = 0;
	byte fmaddr, toaddr;

	while(ch != 0xfe) {
		getR2SerialByte(&ch);
		assert(ch == 0xfe);
	}
	getR2SerialByte(&ch);
	if(ch == 0xfe) 
		getR2SerialByte(&ch); // toaddr

	toaddr = ch;

	getR2SerialByte(&ch); // fmAddr
	fmaddr = ch;

	getR2SerialByte(&cmd);
	sub = 0;

	switch (cmd) {
	case 0x06:
		getR2SerialByte(&(sub));
	default:
		// do nothing
		break;
	}



	numDataBytes = 0;
	getR2SerialByte(&ch);
	while(ch != 0xfd){
		if(numDataBytes < maxDataSize)
			data[numDataBytes++] = ch;
		getR2SerialByte(&ch);
	}
	return 0;
}


int CIVCommand::readFromVlaskiFile(FILE *fp){
	byte ch = 0;
	byte fmaddr, toaddr;

	while(ch != 0xfe) {
		ch = fgetc(fp);
		assert(ch == 0xfe);
	}
	ch = fgetc(fp);
	if(ch == 0xfe) 
		ch = fgetc(fp); // toaddr

	toaddr = ch;

	ch = fgetc(fp);
	fmaddr = ch;

	cmd = fgetc(fp);
	sub = 0;

	switch (cmd) {
	case 0x06:
		sub = fgetc(fp);
	default:
		// do nothing
		break;
	}

	numDataBytes = 0;
	ch = fgetc(fp);
	while(ch != 0xfd){
		if(numDataBytes < maxDataSize)
			data[numDataBytes++] = ch;
		ch = fgetc(fp);
	}
	return 0;

}


#ifndef NDEBUG


int dumpCIVCommand(CIVCommand cmd, int cmdcount) {
    FILE * fp = fopen(DATAFILE, "a");
    assert(fp);
	fprintf (fp,"\n");
	fprintf(fp, "%3d |", cmdcount);
	for(unsigned int i = 0; i < cmd.numDataBytes; i++){
		switch(i) {
		case 6:
		case 22:
		case 38:
			fprintf (fp, " ");
		default:
			fprintf (fp, "%c", cmd.data[i]);
		}
	}
	fprintf (fp,"\n");
	fclose(fp);
	return 0;
}

#endif //ndebug

CR2DataFile::GetFromSerialPort(const char * portName) {

	// start up the serial port

	openR2Port(portName);

	// Tell the R2 to start transmitting data to us
	CIVCommand initcmd;
	initcmd.cmd = 0xe2;
	initcmd.sub = 0x00;
	initcmd.data[0] = 0x21;
	initcmd.data[1] = 0x27;
	initcmd.data[2] = 0x00;
	initcmd.data[3] = 0x01;
	initcmd.numDataBytes = 4;

	initcmd.writeToSerialPort();



	for(int i = 0; i < numDataCmds; i++) {
		int rc = Commands[i].readFromSerialPort();
		assert(rc == 0);

		assert(Commands[i].cmd == 0xe4);
#ifndef NDEBUG
		char buffr[4];
		sprintf(buffr, "%03X", i);
		assert(buffr[0] == Commands[i].data[0]);
		assert(buffr[1] == Commands[i].data[1]);
		assert(buffr[2] == Commands[i].data[2]);
		assert(Commands[i].data[3] == '0');
		assert(Commands[i].data[4] == '1');
		assert(Commands[i].data[5] == '0');
#endif //ndebug

	}


	closeR2Port();


}

int CR2DataFile::GetFromVlaskiFile(const char *filename) {
	FILE * fp = fopen(filename, "r");
	if (!fp) {
//		assert (0 && "fopen failed");
		fprintf(stderr, "Error, can't open %s for read\n", filename);
		return 1;
	}

	for(int i = 0; i < numDataCmds; i++) {
		int rc = Commands[i].readFromVlaskiFile(fp);
		assert(rc == 0);

		assert(Commands[i].cmd == 0xe4);
#ifndef NDEBUG
		char buffr[4];
		sprintf(buffr, "%03X", i);
		assert(buffr[0] == Commands[i].data[0]);
		assert(buffr[1] == Commands[i].data[1]);
		assert(buffr[2] == Commands[i].data[2]);
		assert(Commands[i].data[3] == '0');
		assert(Commands[i].data[4] == '1');
		assert(Commands[i].data[5] == '0');
#endif //ndebug



	}
	fclose(fp);

	return 0;
}


int CR2DataFile::WriteToVlaskiFile(const char *filename) {
	FILE * fp = fopen(filename, "w");
	if (!fp) {
//		assert (0 && "fopen failed");
		fprintf(stderr, "Error, can't open %s for write\n", filename);
		exit(1);
	}

	for(int i = 0; i < numDataCmds; i++) {
		int rc = Commands[i].writeToVlaskiFile(fp);
		assert(rc == 0);

		assert(Commands[i].cmd == 0xe4);
#ifndef NDEBUG
		char buffr[4];
		sprintf(buffr, "%03X", i);
		assert(buffr[0] == Commands[i].data[0]);
		assert(buffr[1] == Commands[i].data[1]);
		assert(buffr[2] == Commands[i].data[2]);
		assert(Commands[i].data[3] == '0');
		assert(Commands[i].data[4] == '1');
		assert(Commands[i].data[5] == '0');
#endif //ndebug



	}
	fclose(fp);

	return 0;
}

int CR2DataFile::WriteToSerialPort(const char *portName) {
	openR2Port(portName);

	CIVCommand initcmd;
	initcmd.cmd = 0xe3;
	initcmd.sub = 0x00;
	initcmd.data[0] = 0x21;
	initcmd.data[1] = 0x27;
	initcmd.data[2] = 0x00;
	initcmd.data[3] = 0x01;
	initcmd.numDataBytes = 4;

	initcmd.writeToSerialPort();

	for(int i = 0; i < numDataCmds; i++) {

		// we want to skip uploading the vfo channels and
		// global options, but upload everything else.
		if(i < numUploadableChannels / 2 || i > 250) {
			int rc = Commands[i].writeToSerialPort();
			assert(rc == 0);
		}

		assert(Commands[i].cmd == 0xe4);
#ifndef NDEBUG
		char buffr[4];
		sprintf(buffr, "%03X", i);
		assert(buffr[0] == Commands[i].data[0]);
		assert(buffr[1] == Commands[i].data[1]);
		assert(buffr[2] == Commands[i].data[2]);
		assert(Commands[i].data[3] == '0');
		assert(Commands[i].data[4] == '1');
		assert(Commands[i].data[5] == '0');
#endif //ndebug



	}

	CIVCommand termcmd;
	termcmd.cmd = 0xe5;
	termcmd.sub = 0x00;
	strcpy((char*)termcmd.data, "Icom Inc.");
	termcmd.numDataBytes = strlen((char*)termcmd.data);

	termcmd.writeToSerialPort();

	closeR2Port();

	return 0;
}




void CR2DataFile::Fixup() {
	// puts in parity checking and such

	char buffr[4];
	int i, j;
	for(i = 0; i < numDataCmds; i++) {
		sprintf(buffr, "%03X", i);
		Commands[i].data[0] = buffr[0];
		Commands[i].data[1] = buffr[1];
		Commands[i].data[2] = buffr[2];
		Commands[i].data[3] = '0';
		Commands[i].data[4] = '1';
		Commands[i].data[5] = '0';

		int sumByte = 0;
		for(j = 0; j < 38; j += 2) {
			sumByte -= value[Commands[i].data[j+1]];
			sumByte -= value[Commands[i].data[j]] << 4;
		}
		sprintf(buffr, "%02X", sumByte & 0xff);
		Commands[i].data[38] = buffr[0];
		Commands[i].data[39] = buffr[1];

		Commands[i].cmd = 0xe4;
		Commands[i].sub = 0x00;
	}
}