/*

    GetR2, program to get ICOM r2 data

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/



// getr2.cpp : Defines the entry point for the console application.
//

#include "r2conf.h"
#include <stdio.h>
#include <assert.h>
#include "r2serial.h"
#include "civ.h"
#include "r2radio.h"
#include "parsebuf.h"


#define R2_EXTERN
#include "getr2.h"



void usage();


int main (int argc, char * argv[])
{
	int i;
	// initialize default values
	char * com_port = COM_PORT;
	char * filename = NULL;
	char * filename2 = NULL;
	
	bool download, upload, echo, verbose;

	download = upload = echo = verbose = false;

	for(i = 1; i < argc; i++) {
		if(argv[i][0] == '-' || argv[i][0] == '/') {
			switch(tolower(argv[i][1])) {
				case 'p': // com port num
					com_port = argv[i+1];
					i++;
					break;
				case 'd': // download
					download = true;
					break;
				case 'u': // upload
					upload = true;
					break;
				case 'e': // echo
					echo = true;
					break;
				case 'v': // verbose
					verbose = true;
					break;
				default:
					printf("Invalid argument - \"%s\"\n", argv[i]);
					usage();
			}
		} else {
			// filename
			if(filename) {
				if(echo && !filename2) {
					filename2 = argv[i];
				} else {
					printf("Invalid argument - \"%s\"\n", argv[i]);
					usage();
				}
			} else {
				filename = argv[i];
			}
		}
	}

	if(!upload && !download && !echo){
		usage();
	}

	if((upload && download) || (upload && echo) || (download && echo)){
		printf("Conflicting arguments\n");
		usage();
	}


// clear the debug log
#ifndef NDEBUG
	FILE * fp = fopen(DATAFILE, "w");
	fclose(fp);
#endif //ndebug

	CR2Radio R2Radio;
	CR2DataFile R2DataFile;


	if(upload) {
		if (!filename) {
			printf("No filename specified\n");
			usage();
		}
		printf("Uploading %s to port %s\n", filename, com_port);

		R2Radio.getFromAsciiFile(filename);
		R2Radio.writeToDataFile(&R2DataFile);
		R2DataFile.WriteToSerialPort(com_port);

	} else if(download) {
		if (!filename) {
			printf("No filename specified\n");
			usage();
		}
		printf("Downloading from port %s to %s", com_port, filename);
		if(verbose) {
			printf(" in verbose mode");
		}
		printf("\n");

		R2DataFile.GetFromSerialPort(com_port);
		R2Radio.getFromDataFile(&R2DataFile);
		R2Radio.writeToAsciiFile(filename, verbose ? 2 : 0);
	
	} else {
		if (!filename || !filename2) {
			printf("Echo needs two filenames\n");
			usage();
		}
		printf("Echoing %s to %s in verbose mode\n", filename, filename2);
		
		R2Radio.getFromAsciiFile(filename);
		R2Radio.writeToAsciiFile(filename2, 2);

	}
	
	return 0;
}

void usage() {
	printf("usage:\n");
	printf("R2Pepper -d filename.txt [-p COMx] [-v]\n");
	printf("R2Pepper -u filename.txt [-p COMx]\n");
	printf("R2Pepper -e filename1.txt filename2.txt\n\n");
	printf("  -d for Download, optional -p to specify serial port name\n");
	printf("                   optional -v to specify verbose file format\n");
	printf("  -u for Upload, optional -p to specify serial port name\n");
	printf("  -e to Echo an input file to an output file, in verbose mode\n");

	exit(1);
}



