
#define PARSEBUF_GLOBAL

#include "parsebuf.h"
#include <stdlib.h>


// all our keywords, defined in alphabetical order 
const keyword keywords[]= {
	{"bank", kBank},
	{"ch", kCh},
	{"dup", kDup},
	{"mode", kMode},
	{"offset", kOffset},
	{"skip", kSkip},
	{"step", kStep},
	{"tone", kTone},
	{"tsql", kTsql}};



CParseBuffer::CParseBuffer(){
	thisLineNum = 0;
}

int CParseBuffer::setDataFromFile(FILE *fp){
	buffpos = 0;
	int ch;

	thisLineNum++;
	
	if(feof(fp))
		return 1;

	while((ch = fgetc(fp)) != EOF && 
		  ch != '\n' && 
		  buffpos < kMaxLineLength) {
		buffer[buffpos++] = ch;
	}
	if(buffpos >= kMaxLineLength) {
		fprintf(stderr, "Error -- line too long at line %d\n", thisLineNum);
		exit (1);
	}
	buffer[buffpos] = '\0';
	buffpos = 0;

	// eliminate comments  -- get rid of data after semicolon
	char *commentpos = strchr(buffer, ';');
	if(commentpos)
		*commentpos = '\0';

	// convert to lower case
	strlwr(buffer);
	
	return 0;
}



void CParseBuffer::getNextToken(CToken *tok){
	
	char * tokenStr = buffer + buffpos;

	// skip blanks
	while(*tokenStr == ' ')
		tokenStr++;

	// find next comma, put a NULL character there, to terminate the string
	char * tokenEnd = strchr(tokenStr, ',');
	if(tokenEnd) {
		*tokenEnd = '\0';
	} else {
		tokenEnd = strchr(tokenStr, '\0');
		tokenEnd[1] = '\0';
	}

	// advance our buffpos
	buffpos = (tokenEnd - buffer) + 1;

	// check to see if we're at EOL
	if(strlen(tokenStr) == 0) {
		tok->type = kEOL;
		tok->data = NULL;
		return;
	}

	// strip trailing blanks
	while (*(tokenEnd - 1) == ' ') {
		tokenEnd--;
		*tokenEnd = '\0';
	}

	//if it has no parens, it must be a frequency
	char * openParenPos = strchr(tokenStr, '(');
	if(openParenPos == NULL) {
		tok->type = kFreq;
		tok->data = tokenStr;
		return;
	}

	// it's got a start paren, put a null there
	*openParenPos = NULL;

	// set data to point to the chars between the parens
	tok->data = openParenPos + 1;
	
	// strip leading blanks in data
	while(*(tok->data) == ' ') 
		tok->data++;

	// strip blanks between keyword and openparen
	while (*(openParenPos - 1) == ' '){
		openParenPos--;
		*openParenPos = NULL;
	}

	// find a close paren
	char * closeParenPos = strchr(tok->data, ')');
	if (!closeParenPos) {
		fprintf(stderr, "Error -- Missing ')' at line %d\n", thisLineNum);
		exit(1);
	}

	// check for data after )
	if (closeParenPos != (tokenEnd - 1)) {
		fprintf(stderr, "Error -- Extra characters after ')' at line %d\n", thisLineNum);
		exit(1);
	}

	// strip trailing blanks in data
    *closeParenPos = NULL;
	while(*(closeParenPos - 1) == ' '){
		closeParenPos--;
		*closeParenPos = NULL;
	}

	

	// binary search of token array

	int first = 0;
	int last = sizeof(keywords)/sizeof(keywords[0]) - 1;
	int mid;

	while (first <= last) {
		mid = (first + last) / 2;
		int cmp = strcmp(tokenStr, keywords[mid].text);
		if(cmp) {
			if(cmp > 0) {
				first = mid + 1;
			} else {
				last = mid - 1;
			}
		} else {
			first = last + 1;
		}
	}

	if(strcmp(tokenStr, keywords[mid].text)){
		fprintf(stderr, "Error -- Invalid keyword \"%s\" at line %d\n", 
			tokenStr, thisLineNum);
		exit(1);
	} 
	tok->type = keywords[mid].token;

	return;
}


