#include "r2conf.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <assert.h>
#include <math.h>
#include "r2serial.h"
#include "civ.h"

#define R2RADIO_GLOBAL
#include "r2radio.h"

static char hexdigits[16] = 
{ '0', '1', '2', '3', '4', '5', '6', '7', 
  '8', '9', 'A', 'B', 'C', 'D', 'E', 'F'};

int value[256] = 
// 00 01 02 03 04 05 06 07 08 09 0a 0b 0c 0d 0e 0f
{   0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, //0
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, //1
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, //2
    0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 0, 0, 0, 0, 0, 0, //3
    0,10,11,12,13,14,15,16,17,18,19,20, 0, 0, 0, 0, //4
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, //5
    0,10,11,12,13,14,15,16,17,18,19,20, 0, 0, 0, 0, //6
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, //7
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, //8
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, //9
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, //a
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, //b
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, //c
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, //d
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, //e
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};//f

static double tones[]= {
67.0, 69.3, 71.9, 74.4, 77.0, 79.7, 82.5, 85.4, 88.5, 91.5, 94.8, 97.4,
100.0, 103.5, 107.2, 110.9, 114.8, 118.8, 123.0, 127.3, 131.8, 136.5, 141.3,
146.2, 151.4, 156.7, 159.8, 162.2, 165.5, 167.9, 171.3, 173.8, 177.3, 179.9, 
183.5, 186.2, 189.9, 192.8, 196.6, 199.5, 203.5, 206.5, 210.7, 218.1, 225.7,
229.1, 233.6, 241.8, 250.3, 254.1};

static unsigned int numValidTones = sizeof(tones)/sizeof(tones[0]);

static double steps[]= {
//                                                             auto ...undefined...  MW9kHz
	5.0, 6.25, 10.0, 12.5, 15.0, 20.0, 25.0, 30.0, 50.0, 100.0, 0., -1., -1., -1., -1., 9.};

static unsigned int numValidSteps = 10;

CChannel::setAbsoluteIndex(const int index){
	bankNo = index / numChannelsPerBank;
	chNo = index % numChannelsPerBank;
}



CChannel::setFromData(const byte * databytes){

/*
   bytes    bits   contents
   0-5             Frequency
   6        3      on if offset is 625 spaced (extra .75, .5, .25 digits displayed)
   6        2      on if freq is 625 spaced
   6        1      On if dup is on
   6        0      On if dup is +, otherwise dup is -
   7-11            Offset


*/
 
	int freq625bit = value[databytes[6]] & 0x08;

    freq.setFromData(databytes, 6, !!freq625bit);

	int offset625bit = value[databytes[6]] & 0x0c;

	offset.setFromData(databytes+7, 5, !!offset625bit);

	// dup
	int dupbits = value[databytes[6]] & 0x03;
	switch(dupbits){
	case 3:
		dup = kDupPlus;
		break;
	case 2:
		dup = kDupMinus;
		break;
	default:
		dup = kDupOff;
		break;
	}

	// tsql
	int tsqlbits = value[databytes[14]] & 0x01;
	switch(tsqlbits){
	case 0:
		tsql = kTsqlOff;
		break;
	case 1:
		tsql = kTsqlOn;
		break;
	}

	// tone
	int tonebits = ((value[databytes[12]] & 0x03) << 4) +
		             value[databytes[13]];

	toneIndex = tonebits;
	if (toneIndex >= numValidTones) {
		toneIndex = numValidTones - 1;
	}

	// mode
	int modebits = (value[databytes[12]] & 0x0c) >> 2;
	switch (modebits){
	case 1:
		mode = kModeWfm;
		break;
	case 2:
		mode = kModeAm;
		break;
	default:
		mode = kModeFm;
	}

	//step
	int stepbits = (value[databytes[15]]);
	stepIndex = stepbits;
	if(steps[stepIndex] < 0.0) {
		stepIndex = 0;
	}

	//skip
	int skipbits = (value[databytes[14]] & 0x0c) >> 2;
	switch(skipbits){
	case 1:
		skip = kSkipP;
		break;
	case 2:
		skip = kSkipOn;
		break;
	default:
		skip = kSkipOff;
		break;
	}
	

}

CChannel::writeToData(byte * databytes){

	byte tempbuff[8] = {0,0,0,0,0,0,0,0};

	int i;

	if(isUndefined()) {
		for(i = 0; i < 8; i++) {
			tempbuff[i] = 0xff;
		}
	} else {

		freq.putToData(tempbuff);

		offset.putToData(tempbuff+3);

		tempbuff[3] &= 0x0f;

		if(freq.is625Spaced())
			tempbuff[3] |= 0x80;
		if(offset.is625Spaced())
			tempbuff[3] |= 0xc0;

		tempbuff[3] |= (dup << 4);

		tempbuff[7] |= (tsql << 4);

		tempbuff[6] |= toneIndex;

		tempbuff[6] |= (mode << 6);

		tempbuff[7] |= stepIndex;

		tempbuff[7] |= (skip << 6);

	}
		
	for(i = 0; i < 8; i++) {
		databytes[2*i] = hexdigits[(tempbuff[i] & 0xf0) >> 4];
		databytes[2*i+1] = hexdigits[tempbuff[i] & 0x0f];
	}
}



CChannel::setFromParseBuffer(CParseBuffer * pb){


    CToken tok;
	int rc;
	bool foundAtLeastOneToken = false;
	bool diamond; 
	bool bankWasSet = false;
	bool chWasSet = false;
	bool offsetWasSet = false;
	bool toneWasSet = false;
	double dblTone, dblStep;
	int first, last, mid;
	char lastChar;

	static CChannel defaultInfo;
	static int defaultToneIndex;
	static CFrequency defaultOffset;

	*this = defaultInfo;
	freq.setAsInt(NullFreqVal);

	// set these to "global" defaults, in case
	// tsql and/or dup aren't set.
	offset.setAsInt(0);
	toneIndex = 8;

	pb->getNextToken(&tok);

	while(tok.type != kEOL) {
		foundAtLeastOneToken = true;
		bool appendedLetter = false;

		switch(tok.type) {

		case kBank:
			diamond = false;
			if(tok.data[0] == '*') {
				diamond = true;
				tok.data++;
			}
			rc = sscanf(tok.data, "%d", &bankNo);
			if (rc != 1) {
				fprintf(stderr, "Invalid Bank Value \"%s\" at line %d\n", tok.data, 
					thisLineNum);
				exit(1);
			}

			if(diamond) {
				bankNo += 4;
			}
			
			if(bankNo < 0 || bankNo >= 10) {
				fprintf(stderr, "Invalid Bank Number %d at line %d\n", bankNo, 
					thisLineNum);
				exit(1);
			}

			bankWasSet = true;

			break;

		case kCh:
			lastChar = tok.data[strlen(tok.data) - 1];
			if(lastChar == 'a' || lastChar == 'b') {
				appendedLetter = true;
				tok.data[strlen(tok.data) - 1] = '\0';
			}
			rc = sscanf(tok.data, "%d", &chNo);
			if (rc != 1) {
				fprintf(stderr, "Invalid Ch Value \"%s\" at line %d\n", tok.data, 
					thisLineNum);
				exit(1);
			}
			
			if((chNo < 0) || (chNo >= 50) || (chNo > 25 && appendedLetter)) {
				fprintf(stderr, "Invalid Channel Number %d at line %d\n", chNo,
					thisLineNum);
				exit(1);
			}

			if(appendedLetter) {
				chNo *= 2;
				chNo += lastChar - 'a';
				bankNo = 8;
				bankWasSet = true;
			}

			chWasSet = true;
			break;

		case kFreq:
			freq.setFromAscii(tok.data);
			break;

		case kDup:
			if(strlen(tok.data) != 1) {
				fprintf(stderr, "Invalid Dup Value \"%s\" at line %d\n", tok.data, 
						thisLineNum);
				exit(1);
			}
			switch(tok.data[0]) {
			case '+':
				dup = kDupPlus;
				break;
			case '-':
				dup = kDupMinus;
				break;
			case '0':
				dup = kDupOff;
				break;
			default:
				fprintf(stderr, "Invalid Dup Value \"%s\" at line %d\n", tok.data, 
						thisLineNum);
				exit(1);
			}
			break;

		case kOffset:
			offset.setFromAscii(tok.data);
			offsetWasSet = true;
			break;

		case kTsql:
			if(!strcmp(tok.data, "off")) {
				tsql = kTsqlOff;
			} else if (!strcmp(tok.data, "on")) {
				tsql = kTsqlOn;
			} else {
				fprintf(stderr, "Invalid Tsql Value \"%s\" at line %d\n", tok.data, 
						thisLineNum);
				exit(1);
			}
			break;

		case kTone:
			rc = sscanf(tok.data, "%lf", &dblTone);

			if(rc != 1) {
				fprintf(stderr, "Bad Tone data \"%s\" at line %d\n", 
					tok.data, thisLineNum);
				exit(1);
			}

			// binary search tone array
			first = 0;
			last = sizeof(tones)/sizeof(tones[0]) - 1;

			while (first <= last) {
				mid = (first + last) / 2;
				if(fabs(dblTone - tones[mid]) > .001) {
					if(dblTone > tones[mid]) {
						first = mid + 1;
					} else {
						last = mid - 1;
					}
				} else {
					first = last + 1;
				}
			}

			if(fabs(dblTone - tones[mid]) > .001){
				fprintf(stderr, "Bad Tone data \"%s\" at line %d\n",
					tok.data, thisLineNum);
#pragma message ("TODO: print list of valid tones")
				exit(1);
			}
			
			toneIndex = mid;

			toneWasSet = true;
			break;

		case kMode:
			if(!strcmp(tok.data, "fm")) {
				mode = kModeFm;
			} else if (!strcmp(tok.data, "wfm")) {
				mode = kModeWfm;
			} else if (!strcmp(tok.data, "am")) {
				mode = kModeAm;
			} else {
				fprintf(stderr, "Invalid Mode Value \"%s\" at line %d\n", tok.data, 
						thisLineNum);
				exit(1);
			}
			break;


		case kStep:
			rc = sscanf(tok.data, "%lf", &dblStep);

			if(rc != 1) {
				// wasn't numeric, see if it was "auto"
				if(strcmp(tok.data, "auto")){
					fprintf(stderr, "Bad Step data \"%s\" at line %d\n", 
						tok.data, thisLineNum);
					exit(1);
				} else {
					stepIndex = kStepAuto;
					break;
				}
			}

			// binary search Step array
			first = 0;
			last = numValidSteps - 1;

			while (first <= last) {
				mid = (first + last) / 2;
				if(fabs(dblStep - steps[mid]) > .001) {
					if(dblStep > steps[mid]) {
						first = mid + 1;
					} else {
						last = mid - 1;
					}
				} else {
					first = last + 1;
				}
			}

			stepIndex = mid;

			if(fabs(dblStep - steps[mid]) > .001){
				// special case: check for 9kHzMW
				if(fabs(dblStep - 9) < .001) {
					stepIndex = kStep9MW;
				} else {
					fprintf(stderr, "Bad Step data \"%s\" at line %d\n",
						tok.data, thisLineNum);
#pragma message ("TODO: print list of valid steps")
					exit(1);
				}
			}

			break;

		case kSkip:
			if(!strcmp(tok.data, "on")) {
				skip = kSkipOn;
			} else if (!strcmp(tok.data, "off")) {
				skip = kSkipOff;
			} else if (!strcmp(tok.data, "p")) {
				skip = kSkipP;
			} else {
				fprintf(stderr, "Invalid Skip Value \"%s\" at line %d\n", tok.data, 
						thisLineNum);
				exit(1);
			}
			break;

						


		default:
			;
		}
		pb->getNextToken(&tok);
	}

	if(foundAtLeastOneToken && freq.isUndefined()) {
		fprintf(stderr, "No frequency found at line %d\n", thisLineNum);
		exit(1);
	}

	// if we switched banks and didn't explicitly set a channel,
	// let's zero the channel
	if(bankWasSet && !(chWasSet))
		chNo = 0;

	// take care of default values for tone and offset
	if(tsql != kTsqlOff) {
		if (!toneWasSet) {
			toneIndex = defaultToneIndex;
		}
		defaultToneIndex = toneIndex;
	}
	
	if(dup != kDupOff) {
		if(!offsetWasSet) {
			offset.setAsInt(defaultOffset.getAsInt());
		}
		defaultOffset.setAsInt(offset.getAsInt());
	}


	// set default channel and bank for next time around
	if(!freq.isUndefined()) {
		defaultInfo = *this;
		defaultInfo.chNo = chNo + 1;
	}

	if(defaultInfo.chNo == numChannelsPerBank) {
		defaultInfo.chNo = 0;
		defaultInfo.bankNo = bankNo + 1;
	}

}


void StripBlanksFromNumber(char * buff) {
// requires there be exactly one string of non-blank chars in the buffer
	
	int firstnonbchar, i;

	for(i = 0; buff[i] == ' '; i++){ 
		/*nothing*/
	}

	firstnonbchar = i;

	for(i = 0; buff[firstnonbchar + i] && (buff[firstnonbchar + i] != ' '); i++){
		buff[i] = buff[firstnonbchar + i];
	}

	buff[i] = NULL;

	for(i = strlen(buff)-1; i && buff[i] == '0'; i--) {
		buff[i] = NULL;
	}
}


	

CChannel::dump(){
#ifndef NDEBUG
    FILE * fp = fopen(DATAFILE, "at");
    assert(fp);

	if (!isUndefined()){
		if(bankNo < 4) {
			fprintf(fp, " %1d-%02d ", bankNo, chNo);
		} else if(bankNo < 8) {
			fprintf(fp, "*%1d-%02d ", bankNo-4, chNo);
		} else if (bankNo == 8) {
			fprintf(fp, " %2d%c ", chNo / 2, (chNo % 2) ? 'b' : 'a');
		} else {
			fprintf(fp, "%1d-%02d ", bankNo, chNo);
		}

		char buff[11];
		freq.putToAscii(buff, 11);
		fprintf(fp, buff);
		fprintf(fp, " ");
		offset.putToAscii(buff,9);
		fprintf(fp, buff);
        fprintf(fp, " ");
		switch(dup) {
		case kDupPlus:
			fprintf(fp, "+D ");
			break;
		case kDupMinus:
			fprintf(fp, "-D ");
			break;
		case kDupOff:
			fprintf(fp, "xD ");
			break;
		}

		switch(tsql) {
		case kTsqlOn:
			fprintf(fp, "TSQL");
			break;
		case kTsqlOff:
			fprintf(fp, "    ");
		}

		fprintf(fp, " %5.1lf ", tones[toneIndex]);

		switch(mode) {
		case kModeFm:
			fprintf(fp, " FM ");
			break;
		case kModeAm:
			fprintf(fp, " AM ");
			break;
		case kModeWfm:
			fprintf(fp, "WFM ");
			break;
		}
		
		switch(stepIndex) {
		case kStepAuto:
			fprintf(fp, " Auto  ");
			break;
		case kStep9MW:
			fprintf(fp, "9KHzMW ");
			break;
		default:
			fprintf(fp, "%6.2lf ", steps[stepIndex]);
		}

		switch(skip) {
		case kSkipOff:
			fprintf(fp, "     ");
			break;
		case kSkipOn:
			fprintf(fp, " Skip");
			break;
		case kSkipP:
			fprintf(fp, "PSkip");
			break;
		}

		fprintf (fp,"\n");
	}
	fclose(fp);
	return 0;

#endif //ndebug
}

CChannel::setValues(int f, int o, modeType m, int s){
	freq.setAsInt(f);
	offset.setAsInt(o);
	mode = m;
	stepIndex = s;
}

CChannel::writeToAsciiFile(FILE * fp, const int verbosity){


	char lineBuf[256];
	char elBuf[30];

	static int previousBank = -1, previousChannel = -1;
	static int lastOffset = 0, lastToneIndex = 8; 
		
	static CChannel lastChannelWritten;


	
	if (isUndefined())
		return 1;
	
	lineBuf[0] = NULL;

	// put "start of bank comment preceeding this channel if the bank
	// has changed
	if(bankNo != previousBank) {
		if(bankNo < 4) {
			sprintf(elBuf, ";\n; Bank %d\n", bankNo);
		} else if(bankNo < 8) {
			sprintf(elBuf, ";\n; Bank *%d\n", bankNo-4);
		} else {
			sprintf(elBuf, ";\n; Scan Limits\n");
			// force output of channel for first scan limit channel
			previousChannel = -5;
		}
		strcat(lineBuf, elBuf);
	}

	// frequency
	char buff[11];
	freq.putToAscii(buff, 11);
	StripBlanksFromNumber(buff);
	sprintf(elBuf, "%s,", buff);
	strcat(lineBuf, elBuf);

	// bank #
	if((verbosity >= 2) || (bankNo != previousBank && bankNo != 8)){
		if(bankNo < 4) {
			sprintf(elBuf, " Bank(%d),", bankNo);
		} else if(bankNo < 8) {
			sprintf(elBuf, " Bank(*%d),", bankNo-4);
		} else {
			sprintf(elBuf, " Bank(%d),", bankNo);
		}
		strcat(lineBuf, elBuf);
	}

	// channel #
	if((verbosity >= 2) || 
	   ((chNo != (previousChannel + 1)) && ((chNo != 0) || bankNo == 8))){
		if(bankNo != 8){
			sprintf(elBuf, " Ch(%d),", chNo);
		} else {
			sprintf(elBuf, " Ch(%d%c),", chNo/2, 'a' + chNo % 2);
		}
		strcat(lineBuf, elBuf);
	}
	previousChannel = chNo;
	previousBank = bankNo;


	// dup
	if(verbosity >= 2 || (dup != lastChannelWritten.dup)) {
		switch(dup) {
		case kDupPlus:
			sprintf(elBuf, " Dup(+),");
		break;
		case kDupMinus:
			sprintf(elBuf, " Dup(-),");
			break;
		case kDupOff:
			sprintf(elBuf, " Dup(0),");
			break;
		}
		strcat(lineBuf, elBuf);
	}

	// offset	
	if(verbosity >= 2 || 
		((offset.getAsInt() != lastOffset) && (verbosity == 1 || dup != kDupOff))){
		offset.putToAscii(buff,9);
		StripBlanksFromNumber(buff);
		sprintf(elBuf, " Offset(%s),", buff);
		strcat(lineBuf, elBuf);
		lastOffset = offset.getAsInt();
	}

	// tsql
	if(verbosity >= 2 || tsql != lastChannelWritten.tsql) {
		switch(tsql) {
		case kTsqlOn:
			sprintf(elBuf, " Tsql(on),");
		break;
		case kTsqlOff:
			sprintf(elBuf, " Tsql(off),");
		}
		strcat(lineBuf, elBuf);
	}

	// Tone
	if(verbosity >= 2 ||
		(((int)toneIndex != lastToneIndex) && 
		 (verbosity == 1 || tsql != kTsqlOff))){
		sprintf(buff, "%5.1lf", tones[toneIndex]);
		StripBlanksFromNumber(buff);
		sprintf(elBuf, " Tone(%s),", buff);
		strcat(lineBuf, elBuf);
		lastToneIndex = toneIndex;
	}

	// Mode
	if(verbosity >= 2 ||
		mode != lastChannelWritten.mode) {
		switch(mode) {
		case kModeFm:
			sprintf(elBuf, " Mode(FM),");
			break;
		case kModeAm:
			sprintf(elBuf, " Mode(AM),");
			break;
		case kModeWfm:
			sprintf(elBuf, " Mode(WFM),");
			break;
		}
		strcat(lineBuf, elBuf);
	}
		
	// Step
	if(verbosity >= 2 || stepIndex != lastChannelWritten.stepIndex) {
		if(stepIndex == kStepAuto) {
			strcat(lineBuf, " Step(Auto),");
		} else {
			sprintf(buff, "%6.2lf", steps[stepIndex]);
			StripBlanksFromNumber(buff);
			sprintf(elBuf, " Step(%s),", buff);
			strcat(lineBuf, elBuf);
		}
	}

	// Skip
	if(verbosity >= 2 || skip != lastChannelWritten.skip) {
		switch(skip) {
		case kSkipOff:
			sprintf(elBuf, " Skip(off),");
			break;
		case kSkipOn:
			sprintf(elBuf, " Skip(on),");
			break;
		case kSkipP:
			sprintf(elBuf, " Skip(p),");
			break;
		}
		strcat(lineBuf, elBuf);
	}
	
	int bufflen = strlen(lineBuf);
	assert(bufflen);
	assert(lineBuf[bufflen-1] == ',');

	lineBuf[bufflen-1] = '\0';
	
	fprintf(fp, "%s\n", lineBuf);

	lastChannelWritten = *this;
	return 0;

}



CR2Radio::CR2Radio() {
	int i;
	for(i = 0; i < numChannels; i++){
		Channels[i].setAbsoluteIndex(i);
	}

	// zero out all options
	for(i = 0; i < 48; i++)
		OptionsDat[i] = 0;

	// fix up some of them
	OptionsDat[ 0] = 1;
	OptionsDat[ 2] = 2;
	OptionsDat[ 3] = 4;
	OptionsDat[ 4] = 1;
	OptionsDat[ 5] = 1;
	OptionsDat[ 8] = 1;
	OptionsDat[ 9] = 2;
	OptionsDat[11] = 1;
	OptionsDat[13] = 1;
	OptionsDat[17] = 5;
	OptionsDat[19] = 1;
	OptionsDat[30] = 17;
	OptionsDat[41] = 0xf0;
	for(i = 42; i < 47; i++)
		OptionsDat[i] = 0xff;
	OptionsDat[47] = 0x3f;


	//set default channel info for bands
	Channels[450].setValues(1620, 0, kModeAm, kStep10);
	Channels[451].setValues(5000, 0, kModeAm, kStep5);
	Channels[452].setValues(51000, 0, kModeFm, kStep20);
	Channels[453].setValues(76000, 0, kModeFm, kStepAuto);
	Channels[454].setValues(118000, 0, kModeAm, kStep25);
	Channels[455].setValues(146010, 600, kModeFm, kStep15);
	Channels[456].setValues(370000, 0, kModeFm, kStep25);
	Channels[457].setValues(440000, 5000, kModeFm, kStep25);
	Channels[458].setValues(850000, 0, kModeFm, kStep25);
	Channels[459].setValues(1295000, 0, kModeFm, kStep20);

	//set default channel info for scan limits
	Channels[400].setValues(540, 0, kModeAm, kStep10);
	Channels[401].setValues(1300, 0, kModeFm, kStep20);

}


CR2Radio::getFromDataFile(const CR2DataFile * dataFile){
	int i, nCmdNum;
	for(i = 0; i < numChannels; i++){

		nCmdNum = i / 2;
		int nPositionInCommand = i % 2;
#ifndef NDEBUG
		if(nPositionInCommand == 0) {
			dumpCIVCommand(dataFile->Commands[nCmdNum], nCmdNum);
		}
#endif // ndebug

		Channels[i].setFromData(&(dataFile->Commands[nCmdNum].
			                 data[6 + 16*nPositionInCommand]));
		Channels[i].dump();
	}

#ifndef NDEBUG
	//dump the rest of the data
	for(i = nCmdNum+1;i < numDataCmds; i++) {
		dumpCIVCommand(dataFile->Commands[i], i);
	}
#endif
}

CR2Radio::writeToDataFile(CR2DataFile * dataFile){

	int i, j, nCmdNum;
	for(i = 0; i < numChannels; i++){

		nCmdNum = i / 2;
		int nPositionInCommand = i % 2;
		if(nPositionInCommand == 0) {
			dataFile->Commands[nCmdNum].numDataBytes = maxDataSize;
			for(j = 0; j < maxDataSize; j++)
				dataFile->Commands[nCmdNum].data[j] = 'F';
		}

		Channels[i].writeToData(&(dataFile->Commands[nCmdNum].
			                 data[6 + 16*nPositionInCommand]));
	}
	nCmdNum++;

	//dump the rest of the data
	for(i = 0;i < 3; i++) {
		for(j = 0; j < 16; j++)
			sprintf((char*)(dataFile->Commands[nCmdNum].data) + 6 + 2*j,
			"%02X", OptionsDat[j + 16*i]);
		dataFile->Commands[nCmdNum++].numDataBytes = maxDataSize;
//		sprintf((dataFile->Commands[nCmdNum].data) + 6, ");
//		dumpCIVCommand(dataFile->Commands[i], i);
	}

	while(nCmdNum < 252) {
		strcpy((char*)dataFile->Commands[nCmdNum].data + 6, 
			"00000000000000000000000000000000");
		dataFile->Commands[nCmdNum++].numDataBytes = maxDataSize;
	}

	strcpy((char*)dataFile->Commands[250].data + 6, 
		"20202020202020202020202020202020");
	strcpy((char*)dataFile->Commands[251].data + 6, 
		"49636F6D436C6F6E65466F726D617433");

	dataFile->Fixup();
}



CR2Radio::writeToAsciiFile(const char * fn, const int verbosity){
	FILE * fp = fopen(fn, "wt");
	if (!fp) {
		assert(0 && "can't open file");
		fprintf(stderr, "Can't open %s for write\n", fn);
	}

	time_t secs_now;
	char str[80];
	tzset();
	time (&secs_now);
	strftime(str, 80, "%a, %b %d, %Y at %X", localtime(&secs_now));

	fprintf(fp, ";\n; Written by %s on %s\n", ID_STRING, str);
	


	int i;
	for(i = 0; i < numUploadableChannels; i++){
		Channels[i].writeToAsciiFile(fp, verbosity);
	}

	fclose(fp);
}

CR2Radio::getFromAsciiFile(const char * fn) {
	FILE * fp = fopen(fn, "rt");

	int weSetChannel[numChannels];
	int i;

	for(i = 0; i < numChannels; i++)
		weSetChannel[i] = 0;

	CParseBuffer parsebuffer;
	while(parsebuffer.setDataFromFile(fp) == 0){
		CChannel chan;
		chan.setFromParseBuffer(&parsebuffer);
		if(!chan.isUndefined()){
			i = chan.AbsoluteIndex();
			if (i >= 0 && i < numChannels) {
				if(weSetChannel[i]) {
					assert(Channels[i].AbsoluteIndex() == i);
					fprintf(stderr, 
						"Warning: Bank %d Ch %d was defined at line %d.  Second definition at line %d\n",
						Channels[i].bank(), Channels[i].ch(), weSetChannel[i], thisLineNum);
				}
				Channels[i] = chan;
				weSetChannel[i] = thisLineNum;
			}
		}
	}

}

CFrequency::setFromData(const byte * dataBytes, const unsigned int len, 
						const bool is625spaced){
	freq = 0;

	if(value[dataBytes[0]] < 10 && 
	   value[dataBytes[1]] < 10 && 
	   value[dataBytes[2]] == 15) {
		// we're in the MW band, special case;
		bIs9KHzMW = true;
		int numSteps = 10 * value[dataBytes[0]] + 
			value[dataBytes[1]];
		freq = 495 + 9 * numSteps;
	} else {
		// normal frequency format
		bIs9KHzMW = false;
		bIs625Spaced = is625spaced;
		int mult = 1;
		for(unsigned int i = 1; i < len; i++) {
			mult *= 10;
		}
		int magnitude = mult;
		for(i = 0; i < len; i++) {
			freq += value[dataBytes[i]] * mult;
			mult /= 10;
		}
		if (freq >= 16 * magnitude) 
			freq = NullFreqVal;
	}
}



CFrequency::setFromAscii(const char *buff) {

	int rc;
	double dblFreq;

	rc = sscanf(buff, "%lf", &dblFreq);

	if(rc != 1) {
		fprintf(stderr, "Bad frequency or offset data \"%s\" at line %d\n");
		exit(1);
	}

	dblFreq *= 1000; // convert to kHz

	freq = (int)(dblFreq + .1); // the .1 is for roundoff
	
	bIs625Spaced = ((dblFreq - freq) > .1);

	int lastdigit = freq % 10;

	bIs9KHzMW = ((lastdigit != 0) && freq < 1620 && freq >= 495 &&
		((freq - 495) % 9 == 0) );

}

void intToBcd(const int i, byte * bcd) {
	int nib1 = i / 10;
	int nib2 = i % 10;
	*bcd = (nib1 << 4) | nib2;
}

CFrequency::putToData(byte * buffer){

	
	if(isUndefined()) {
		buffer[0] = buffer[1] = buffer[2] = 0xff;
	} else if(bIs9KHzMW) {
		// we're in the MW band, special case;
		assert(freq <= 1620 && freq >= 495);
		int numSteps = (freq - 495) / 9;
		intToBcd(numSteps, buffer);
		buffer[1] = buffer[2] = 0xff;
	} else {
		// normal frequency format
		int byte1, byte2, byte3;
		byte3 = freq % 100;
		byte2 = (freq / 100) % 100;
		byte1 = freq / 10000;
		intToBcd(byte1, buffer);
		intToBcd(byte2, buffer + 1);
		intToBcd(byte3, buffer + 2);
	}
}



CFrequency::putToAscii(char * buff, int bufflen) const{

	// we need space for three digits after decimal, plus
	// possible two more for fractional positions.
	// This means ten places for a frequency up to gigahertz,
	// or eight places for an offset (plus null char).
	assert(freq < 9000000);
	assert(bufflen == 9 || bufflen == 11);
	assert((freq < 100000) || (bufflen >= 11));
	if(!isUndefined()) {
		int mHz = freq / 1000;
		int kHz = freq % 1000;
		int lastdigit = kHz % 10;
		if(bufflen == 9) {
			sprintf(buff, "%2d.%03d", mHz, kHz);
		} else {
			sprintf(buff, "%4d.%03d", mHz, kHz);
		}
		if (bIs625Spaced) {
			switch(lastdigit) {
			case 0:
			case 5:
				strcat(buff, "  ");
				break;
			case 1:
			case 6:
				strcat(buff, "25");
				break;
			case 2:
			case 7:
				strcat(buff, "5 ");
				break;
			case 3:
			case 8:
				strcat(buff, "75");
				break;
			default:
				strcat(buff, "xx");
			}
		} else {
			strcat(buff, "  ");
		}
	} else {
		for(int i = 0; i < bufflen; i++) {
			buff[i] = ' ';
		}
		buff[bufflen-1] = '\0';
	}
}

