#include "parsebuf.h"


const int numChannels = 460;
const int numChannelsPerBank = 50;
const int numUploadableChannels = 450;
const int NullFreqVal = -1;

typedef enum{kDupOff, kDupMinus = 2, kDupPlus} dupType;
typedef enum{kModeFm, kModeWfm, kModeAm} modeType;
enum {kStep5 = 0,
      kStep625,
	  kStep10,
	  kStep125,
	  kStep15,
	  kStep20,
	  kStep25,
	  kStep30,
	  kStep50,
	  kStep100,
	  kStepAuto = 10, 
	  kStep9MW = 15};


class CFrequency {
public:
	CFrequency() {freq = NullFreqVal; bIs9KHzMW = false; bIs625Spaced = false;};
	setFromData (const byte * databytes, const unsigned int len, 
		         const bool is625spaced);
	putToData (byte * databuff);
	putToAscii (char * buff, int bufflen) const;
	setFromAscii(const char *buff);
	bool isUndefined() const {return freq == NullFreqVal;};
	bool is9KHzMW() const {return bIs9KHzMW;};
	bool is625Spaced() const {return bIs625Spaced;};
	int getAsInt() const {return freq;};
	void setAsInt(const int f) {freq = f;};
private:
	int freq; // in kHz
	bool bIs9KHzMW;
	bool bIs625Spaced;
};



class CChannel {
public:
	CChannel(int f=NullFreqVal);
	setValues(int f, int o, modeType mode, int step);
	setFromParseBuffer(CParseBuffer * pb);
	setFromData(const byte * databytes);
	writeToData(byte * databytes);
	writeToAsciiFile(FILE *fp, int verbosity);
	setAbsoluteIndex(const int i);
	int AbsoluteIndex() const {return numChannelsPerBank*bankNo + chNo;};
	bool isUndefined() const{return freq.isUndefined();};
	dump();
	int bank() const {return bankNo;};
	int ch() const {return chNo;};
private:
	int bankNo; // 0-4, *0-*4, etc.
	int chNo;
	CFrequency freq;
	CFrequency offset;
	dupType dup;
	enum{kTsqlOff, kTsqlOn} tsql;
	unsigned int toneIndex;
	modeType mode;
	unsigned int stepIndex;
	enum{kSkipOff, kSkipP, kSkipOn} skip;
};


inline CChannel::CChannel(int f) {
	bankNo = 0;
	chNo = 0;
	if(f != NullFreqVal)
		freq.setAsInt(f);
	offset.setAsInt(0);
	dup = kDupOff;
	tsql = kTsqlOff;
	toneIndex = 8;
	mode = kModeFm;
	stepIndex = kStep15;
	skip = kSkipOff;
}

class CR2Radio {
public:
	CR2Radio();
	getFromDataFile(const CR2DataFile * dataFile);
	writeToDataFile(CR2DataFile * dataFile);
	writeToAsciiFile(const char * filename, const int verbosity);
	getFromAsciiFile(const char * fn);
private:
	CChannel Channels[numChannels];
    byte OptionsDat[48];
};